@@ -8,10 +8,11 @@ HTTP response API.
 from typing import Union
 
 from attr import Factory, attrib, attrs
-from attr.validators import instance_of, provides
+from attr.validators import instance_of
 from tubes.itube import IFount
 from zope.interface import implementer
 
+from ._attrs_zope import provides
 from ._imessage import IHTTPHeaders, IHTTPResponse
 from ._message import MessageState, bodyAsBytes, bodyAsFount, validateBody
 
