@@ -1,16 +1,16 @@
-from typing import Iterable, List, Text, Tuple
+from typing import Iterator, Sequence, Tuple, cast
 
 from hyperlink import DecodedURL
-
 from treq.testing import StubTreq
+from zope.interface import Interface
 
+from twisted.python.components import Componentized
 from twisted.trial.unittest import SynchronousTestCase
 from twisted.web.http_headers import Headers
 from twisted.web.iweb import IRequest
 
-from zope.interface import Interface
-
 from klein import Klein, RequestComponent, RequestURL, Requirer, Response
+from klein.interfaces import IRequiredParameter
 
 
 class BadlyBehavedHeaders(Headers):
@@ -19,12 +19,11 @@ class BadlyBehavedHeaders(Headers):
     getAllRequestHeaders.
     """
 
-    def getAllRawHeaders(self):
-        # type: () -> Iterable[Tuple[bytes, List[bytes]]]
+    def getAllRawHeaders(self) -> Iterator[Tuple[bytes, Sequence[bytes]]]:
         """
         Don't return a host header.
         """
-        for key, values in super(BadlyBehavedHeaders, self).getAllRawHeaders():
+        for key, values in super().getAllRawHeaders():
             if key != b"Host":
                 yield (key, values)
 
@@ -34,14 +33,16 @@ requirer = Requirer()
 
 
 @requirer.require(
-    router.route("/hello/world", methods=["GET"]), url=RequestURL()
+    router.route("/hello/world", methods=["GET"]),
+    # typing note: https://github.com/Shoobx/mypy-zope/issues/39
+    url=cast(IRequiredParameter, RequestURL()),
 )
-def requiresURL(url):
-    # type: (DecodedURL) -> Text
+def requiresURL(url: DecodedURL) -> str:
     """
     This is a route that requires a URL.
     """
-    return url.child(u"hello/ world").asText()
+    text: str = url.child("hello/ world").asText()
+    return text
 
 
 class ISample(Interface):
@@ -51,20 +52,18 @@ class ISample(Interface):
 
 
 @requirer.prerequisite([ISample])
-def provideSample(request):
-    # type: (IRequest) -> None
+def provideSample(request: IRequest) -> None:
     """
     This requirer prerequisite installs a string as the provider of ISample.
     """
-    request.setComponent(ISample, "sample component")
+    cast(Componentized, request).setComponent(ISample, "sample component")
 
 
 @requirer.require(
     router.route("/retrieve/component", methods=["GET"]),
     component=RequestComponent(ISample),
 )
-def needsComponent(component):
-    # type: (Text) -> Text
+def needsComponent(component: str) -> str:
     """
     This route requires and returns an L{ISample}.
     """
@@ -72,8 +71,7 @@ def needsComponent(component):
 
 
 @requirer.require(router.route("/set/headers"))
-def someHeaders():
-    # type: () -> Response
+def someHeaders() -> Response:
     """
     Set some response attributes.
     """
@@ -89,8 +87,7 @@ class RequireURLTests(SynchronousTestCase):
     Tests for RequestURL() required parameter.
     """
 
-    def test_requiresURL(self):
-        # type: () -> None
+    def test_requiresURL(self) -> None:
         """
         When RequestURL is specified to a requirer, a DecodedURL object will be
         passed in to the decorated route.
@@ -107,8 +104,7 @@ class RequireURLTests(SynchronousTestCase):
             response, "https://example.com/hello/world/hello%2F%20world"
         )
 
-    def test_requiresURLNonStandardPort(self):
-        # type: () -> None
+    def test_requiresURLNonStandardPort(self) -> None:
         """
         When RequestURL is specified to a requirer, a DecodedURL object will be
         passed in to the decorated route.
@@ -125,8 +121,7 @@ class RequireURLTests(SynchronousTestCase):
             response, "http://example.com:8080/hello/world/hello%2F%20world"
         )
 
-    def test_requiresURLBadlyBehavedClient(self):
-        # type: () -> None
+    def test_requiresURLBadlyBehavedClient(self) -> None:
         """
         requiresURL will press on in the face of badly-behaved client code.
         """
@@ -150,8 +145,7 @@ class RequireComponentTests(SynchronousTestCase):
     Tests for RequestComponent.
     """
 
-    def test_requestComponent(self):
-        # type: () -> None
+    def test_requestComponent(self) -> None:
         """
         Test for requiring a component installed on the request.
         """
@@ -170,15 +164,16 @@ class ResponseTests(SynchronousTestCase):
     Tests for L{klein.Response}.
     """
 
-    def test_basicResponse(self):
-        # type: () -> None
+    def test_basicResponse(self) -> None:
         """
         Since methods decorated with C{@require} don't receive the request and
         can't access it to set headers and response codes, instead, they can
         return a Response object that has those attributes.
         """
         response = self.successResultOf(
-            StubTreq(router.resource()).get("https://example.com/set/headers",)
+            StubTreq(router.resource()).get(
+                "https://example.com/set/headers",
+            )
         )
         self.assertEqual(response.code, 209)
         self.assertEqual(
