@@ -1,4 +1,4 @@
-# Copyright (c) 2011-2019. See LICENSE for details.
+# Copyright (c) 2011-2021. See LICENSE for details.
 
 """
 Tests for L{klein.test._trial}.
@@ -7,7 +7,6 @@ Tests for L{klein.test._trial}.
 from zope.interface import Interface, implementer
 
 from ._trial import TestCase
-from .._typing import ifmethod
 
 
 __all__ = ()
@@ -23,31 +22,27 @@ class TestCaseTests(TestCase):
         Frobbable object.
         """
 
-        @ifmethod
-        def frob():
-            # type: () -> None
+        def frob() -> None:
             """
             Frob the object.
             """
 
     @implementer(IFrobbable)
-    class Frobbable(object):
+    class Frobbable:
         """
         Implements L{IFrobbable}.
         """
 
-        def frob(self):
-            # type: () -> None
+        def frob(self) -> None:
             pass
 
     @implementer(IFrobbable)
-    class NotFrobbable(object):
+    class NotFrobbable:  # type: ignore[misc]  # …intentional for test
         """
         Does not implement L{IFrobbable}, despite declaring.
         """
 
-    def test_assertProvidesPass(self):
-        # type: () -> None
+    def test_assertProvidesPass(self) -> None:
         """
         L{TestCase.assertProvides} does not raise when C{interface} is provided
         by C{obj}.
@@ -56,8 +51,7 @@ class TestCaseTests(TestCase):
         self.assertProvides(self.IFrobbable, frobbable)
         frobbable.frob()  # Coverage
 
-    def test_assertProvidesFail(self):
-        # type: () -> None
+    def test_assertProvidesFail(self) -> None:
         """
         L{TestCase.assertProvides} does not raise when C{interface} is not
         provided by C{obj}.
